#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.


#Functional alias module

import logging
import json
from sys import exc_info
import requests
import tornado.web
import tornado.escape
from dbod.api.base import *
from dbod.config import config

class FunctionalAlias(tornado.web.RequestHandler):

    """
    This is the handler of **/instance/alias/<database name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table that is used for this endpoint is called *functional_aliases* and provides information for the functional alias association with an instance.
    * The columns of this table are like that:

    +------------+------------+----------------------+
    |  dns_name  |  name   |         alias        |
    +============+============+======================+
    | dbod-dns42 | dbod-db42  | dbod-alias42.cern.ch |
    +------------+------------+----------------------+

        * The *dns_name* is used internally. They point to a list of virtual IP addresses and each bound to a host
        * The *name* is the name of the database instance
        * The *alias* is the alias given in order the database to be accessed with that name

    * There is a pool of *dns_names* in this table and the other 2 columns are *NULL* in the begining

    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post`
    * :func:`delete` 

    .. note::

      You need to provide a <*username*> and a <*password*> to use
      :func:`post` and :func:`delete` methods or to provide manually the *Authorization* header.

    """

    url = config.get('postgrest', 'functional_alias_url')

    def get(self, name):

        """
        The *GET* method returns the database name's *alias* and *dns name*.
        (No any special headers for this request)

        :param name: the instance name which is given in the url
        :type name: str
        :rtype: json -- the response of the request
        :raises: HTTPError - when the requested database name does not exist or if there is an internal error 
        
        """
        response = requests.get(config.get('postgrest', 'functional_alias_url') + "?name=eq." + name )
        if response.ok:
            data = response.json()
            if data:
                self.write({'response' : data})
                self.set_status(OK)
            else:
                logging.error("Functional alias not found for instance: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Error fetching functional alias: " + name)
            raise tornado.web.HTTPError(NOT_FOUND)


    @http_basic_auth
    def post(self, id):

        """
        The *POST* method inserts a new *instance name* and its *alias* into the database. It
        adds the functional alias association for an instance.

        The *dns name* is chosen automatically from a pool; so, in the background this method 
        actually updates the *instance name* and *alias* fields, which were *NULL* in the
        begining.

        .. note::

            This method is not successful:

            * if the *instance name* already exists
            * if there are no any *dns names* available
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param id: the new instance id which is given in the url
        :type id: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right
        :request body: <instace_id>=instance_id alias=<alias> - the alias to be inserted for the given *database name* which is given in the *body* of the request

        """
        logging.debug(self.request.body)
        try:
            functional_alias = {'in_json': json.loads(self.request.body)}

            # Insert the instance in database using PostREST
            response = requests.post(config.get('postgrest', 'insert_functional_alias_url'), json=functional_alias, headers={'Prefer': 'return=representation'})
            if response.ok:
                logging.info("Created functional Alias " + functional_alias["in_json"]["instance_id"])
                logging.debug(response.text)
                self.set_status(CREATED)
            else:
                logging.error("Error inserting the functional alias: " + response.text)
                raise tornado.web.HTTPError(response.status_code)
        except:
            logging.error("Argument not recognized or not defined.")
            logging.error("Try adding header 'Content-Type:application/x-www-form-urlencoded'")
            raise tornado.web.HTTPError(BAD_REQUEST)



    @http_basic_auth
    def delete(self, id):
        """
        The *DELETE* method deletes or else asssigns to *NULL* the *database name* and 
        *alias* fields. It removes the functional alias association for an instance.

        .. note::
            
            * If the *database name* doesn't exist it doesn't do anything
            * You have to be authorized to use this method
        
        :param id: the new database name which is given in the url
        :type id: str
        :raises: HTTPError - when the deletion is not successful

        """
        functional_alias = {'id': id}
        logging.debug('Arguments:' + str(self.request.arguments))
        response = requests.post(config.get('postgrest', 'delete_functional_alias_url'), json=functional_alias, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Delete functional alias " + functional_alias["id"])
            logging.debug(response.text)
            self.set_status(CREATED)
        else:
            logging.error("Error deleting functional alias: " + response.text)
            raise tornado.web.HTTPError(response.status_code)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.


#Functional alias module

import logging
import json
import requests
import tornado.web
import tornado.escape
from dbod.api.base import *
from dbod.config import config

class Host(tornado.web.RequestHandler):
   
    """
    This is the handler of **/host/names/<name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table that is used for this endpoint is called *host* and provides information for the functional alias association with an instance.
    * The columns of this table are like that:

    +----+-----------+--------+
    | id |  name     | memory |
    +====+===========+========+
    | 42 | dbod-db42 |  1024  |
    +----+-----------+--------+

        * The *name* in this example is the hostname of a node
        * The *memory* is the memory of the node expressed as integer in megabytes

    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post`
    * :func:`put`
    * :func:`delete` 

    .. note::

      You need to provide a <*username*> and a <*password*> to use
      :func:`post`, :func:`put`, and :func:`delete` methods or to 
      provide manually the *Authorization* header.

    """

    url = config.get('postgrest', 'host_url')

    def get(self, name):

        """
        The *GET* method returns the host's memory according to the example given above.
        (No any special headers for this request)

        :param name: the hostname which is given in the url
        :type name: str
        :rtype: json -- the response of the request
        :raises: HTTPError - when the requested name does not exist or if there is an internal 
        error or if the response is empty
        """

        response = requests.get(config.get('postgrest', 'host_url') + "?name=eq." + name)
        if response.ok:
            data = response.json()
            if data:
                self.write({'response' : data})
                self.set_status(OK)
            else:
                logging.error("host metadata not found: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Host metadata not found: " + name)
            raise tornado.web.HTTPError(NOT_FOUND)

    @http_basic_auth
    def post(self, id):

        """
        The *POST* method inserts a new *name* and its *memory* size according to the example above. 
        You don't have to specify anything about the *id* since this field should be specified as *serial*.

        .. note::

            This method is not successful:

            * if the *name* already exists
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param id: the new id which is given in the url
        :type id: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right orthere is an internal error
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """

        logging.debug(self.request.body)

        try:
            host = {'in_json': json.loads(self.request.body)}

            # Insert the instance in database using PostREST
            response = requests.post(config.get('postgrest', 'insert_host_url'), json=host, headers={'Prefer': 'return=representation'})
            if response.ok:
                logging.debug(response.text)
                self.set_status(CREATED)
            else:
                logging.error("Error inserting the host: " + response.text)
                raise tornado.web.HTTPError(response.status_code)
        except:
            logging.error("Argument not recognized or not defined.")
            logging.error("Try adding header 'Content-Type:application/x-www-form-urlencoded'")
            raise tornado.web.HTTPError(BAD_REQUEST)

	
    @http_basic_auth   
    def put(self, id):
        """
        The *PUT* method updates the *host*.


        .. note::

            In order to be able to use *PUT* method possibly you have to specify this header:
            'Content-Type: application/x-www-form-urlencoded'.

            This method is not successful:

            * if the *host* does not exist
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param name: the new name which is given in the url
        :type name: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right orthere is an internal error
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """
        logging.debug(self.request.body)
        host = {'id': id, 'in_json': json.loads(self.request.body)}
        response = requests.post(config.get('postgrest', 'update_host_url'), json=host, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Update host: " + id)
            logging.debug(response.text)
            self.set_status(CREATED)
        else:
            logging.error("Error updating the host: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

    @http_basic_auth
    def delete(self, id):
        """
        The *DELETE* method deletes an entry from the table given the *name* in the url.
        You don't have to specify anything about the *id*.


        .. note::
            
            This method is not successful:

            * if the *name* does not exist
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param name: the new name which is given in the url
        :type name: str
        :raises: HTTPError - when there is an internal error or the requested name does not exist
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """
        logging.debug(self.request.body)
        host = {'id': id}

        # Insert the instance in database using PostREST
        response = requests.post(config.get('postgrest', 'delete_host_url'), json=host, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Delete host " + host["id"])
            logging.debug(response.text)
            self.set_status(CREATED)
        else:
            logging.error("Error delete the host: " + response.text)
            raise tornado.web.HTTPError(response.status_code)




#!/usr/bin/env python
"""Testing functional alias endpoint"""
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import json
import unittest
import base64
import requests
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from dbod.api.api import handlers
from dbod.config import config

class FunctionalAliasTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing functional alias with nosetest"""


    name_test = "dbod42"
    alias_test = "dbod-dbod-42.cern.ch"
    authentication = "basic " + base64.b64encode(config.get('api', 'user') + ":" + config.get('api', 'pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers)

    @timeout(5)
    def test_get_single_alias_by_name(self):
        """test for getting the right data"""
        print "test_get_single_alias_by_name"
        name = 'dbod01'
        response = self.fetch("/api/v1/instance/alias/%s" %(name))
        data = json.loads(response.body)["response"]
        self.assertEquals(response.code, 200)

        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["alias"], "dbod-dbod-01.cern.ch")
        self.assertTrue(data[0]["dns_name"] != None)
        self.assertEquals(response.headers['Content-Type'], 'application/json; charset=UTF-8')


    @timeout(5)
    def test_get_invalid_instance_name(self):
        """test when the instance name does not exist"""
        print "test_get_invalid_instance_name"
        response = self.fetch("/api/v1/instance/alias/%s" %(self.name_test))
        data = response.body
        self.assertEquals(response.code, 404)
        self.assertEquals(len(data), 69)
        self.assertEquals(response.headers['Content-Type'], 'text/html; charset=UTF-8')


    # @timeout(5)
    def test_novalid_instance(self):
         """test when the given instance does not exist"""
         response = self.fetch("/api/v1/instance/alias/some_instance")
         self.assertEquals(response.code, 404)


    @timeout(5)
    def test_create_delete_a_valid_alias(self):
        """test insert a new alias (dns_name is available) and delete the data inserted"""
        print "test_post_valid_request"

        funtional_alias = """{"instance_id" : "5", "alias" : "dbod-dbod-05.cern.ch" }"""

        response = self.fetch("/api/v1/instance/alias/create", method="POST", headers={'Authorization': self.authentication}, body=funtional_alias)
        self.assertEquals(response.code, 201)

        # confirm you get back the right data
        response = self.fetch("/api/v1/instance/alias/dbod05")
        data = json.loads(response.body)["response"]
        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["alias"], "dbod-dbod-05.cern.ch")
        self.assertTrue(data[0]["dns_name"] != None)

        # delete what has been created
        self.fetch("/api/v1/instance/alias/" + str(data[0]["id"]), headers={'Authorization': self.authentication}, method="DELETE")

    # @timeout(5)
    def test_post_duplicate(self):
        """test when there is a request to insert an instance which already exists"""
        print "test_post_duplicate"
        funtional_alias = """{"instance_id" : "1", "alias" : "dbod-dop-01.cern.ch" }"""
        response = self.fetch("/api/v1/instance/alias/dbod01", method="POST", headers={'Authorization': self.authentication}, body=funtional_alias)
        self.assertEquals(response.code, 201)

        # confirm you get back the right data
        response = self.fetch("/api/v1/instance/alias/dbod01")
        data = json.loads(response.body)["response"]
        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["alias"], "dbod-dbod-01.cern.ch")

    # @timeout(5)
    def test_post_no_dns(self):
        """test when there are no any dns available"""
        funtional_alias = """{"instance_id" : "5", "alias" : "dbod-dbod-05.cern.ch" }"""

        response = self.fetch("/api/v1/instance/alias/create", method="POST", headers={'Authorization': self.authentication}, body=funtional_alias)
        self.assertEquals(response.code, 201)


        funtional_alias = """{"instance_id" : "6", "alias" : "dbod-dbod-06.cern.ch" }"""

        response = self.fetch("/api/v1/instance/alias/create", method="POST", headers={'Authorization': self.authentication}, body=funtional_alias)
        self.assertEquals(response.code, 201)

        # confirm you get back the right data
        response = self.fetch("/api/v1/instance/alias/dbod06")
        self.assertEquals(response.code, 404)

        # delete what has been created
        self.fetch("/api/v1/instance/alias/5", headers={'Authorization': self.authentication}, method="DELETE")



    @timeout(5)
    def test_post_no_valid_argument(self):
        """test if the provided argument is not valid"""
        funtional_alias = """{"instance_id" : "10"}"""

        response = self.fetch("/api/v1/instance/alias/create", method="POST", headers={'Authorization': self.authentication}, body=funtional_alias)
        self.assertEquals(response.code, 400)




# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import unittest
from types import *
import json
import logging
import sys
import requests

import base64
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from dbod.api.api import handlers
from dbod.config import config
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class HostTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing Host endpoint with nosetest"""
    
    authentication = "basic " + base64.b64encode(config.get('api', 'user') + ":" + config.get('api', 'pass'))

    def get_app(self):
        return tornado.web.Application(handlers)

    @timeout(5)
    def test_get_valid_name(self):
        """ test a successful get method with a valid given name"""

        # Check the data for the given host
        print "test_get_host_metadata"
        response = self.fetch("/api/v1/host/host01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["name"], "host01")
        self.assertEquals(data[0]["memory"], 12)

    @timeout(5)
    def test_create_delete_host(self):

        """test for create and delete a host with the right data"""

        host = """{"name": "host05", "memory": "512" }"""
        print "test_create_delete_host"

        # Create the host
        response = self.fetch("/api/v1/host/create", method='POST', headers={'Authorization': self.authentication}, body=host)
        self.assertEquals(response.code, 201)

        # Check for this new host
        response = self.fetch("/api/v1/host/host05")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["name"], "host05")
        self.assertEquals(data[0]["memory"], 512)


        # Delete the created host
        response = self.fetch("/api/v1/host/" + str(data[0]["id"]) , method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 201)

        # Check again, the metadata should be empty
        response = self.fetch("/api/v1/names/host05")
        self.assertEquals(response.code, 404)

    @timeout(5)
    def test_insert_duplicate_host(self):

        """test for create an already existing host"""
        host = """{"name": "host01", "memory": "512" }"""

        # Create the host
        response = self.fetch("/api/v1/host/create", method='POST', headers={'Authorization': self.authentication}, body=host)
        self.assertEquals(response.code, 400)



    @timeout(5)
    def test_update_host(self):
        """test for update a host"""
        current_host = """{"memory": "12" }"""
        new_host = """{"memory": "1200" }"""

        response = self.fetch("/api/v1/host/1" , method='PUT', headers={'Authorization': self.authentication}, body=new_host)
        self.assertEquals(response.code, 201)

        # Check the cluster update
        response = self.fetch("/api/v1/host/host01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["memory"], 1200)

        # Restore the instance
        response = self.fetch("/api/v1/host/1", method='PUT', headers={'Authorization': self.authentication}, body=current_host)
        self.assertEquals(response.code, 201)

        # Check the cluster update (restore)
        response = self.fetch("/api/v1/host/host01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["memory"], 12)


    @timeout(5)
    def test_get_invalid_cluster(self):
        print "test_get_invalid_host"
        response = self.fetch("/api/v1/host/invalid")
        self.assertEquals(response.code, 404)


